package com.qmth.distributed.print.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.distributed.print.business.bean.params.ObeCourseOutlineParam;
import com.qmth.distributed.print.business.bean.result.CourseWeightSettingStatusResult;
import com.qmth.distributed.print.business.bean.result.ObeCourseOutlineResult;
import com.qmth.distributed.print.business.entity.ObeCourseOutline;
import com.qmth.distributed.print.business.enums.CourseSettingTypeEnum;

/**
 * <p>
 * 课程大纲表 服务类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-06-05
 */
public interface ObeCourseOutlineService extends IService<ObeCourseOutline> {

    /**
     * 课程大纲分页查询
     *
     * @param professionalId    专业id
     * @param cultureProgramId  培养方式id
     * @param courseId          课程id
     * @param courseOutlineName 课程大纲名称
     * @param pageNumber        分页页码
     * @param pageSize          分页容量
     * @return 分页数据
     */
    IPage<ObeCourseOutlineResult> findObeCourseOutlinePage(Long professionalId, Long cultureProgramId, Long courseId,
            String courseOutlineName, int pageNumber, int pageSize);

    /**
     * 课程大纲保存
     *
     * @param obeCourseOutlineParams 课程大纲
     */
    void saveObeCourseOutline(ObeCourseOutlineParam obeCourseOutlineParams);

    /**
     * 课程大纲删除
     *
     * @param id id
     */
    void removeObeCourseOutline(Long id);

    /**
     * 清除课程目标、评价、权重设置
     *
     * @param obeCourseOutlineId    课程大纲id
     * @param courseSettingTypeEnum 课程设置类型
     */
    void clearCourseSetting(Long obeCourseOutlineId, CourseSettingTypeEnum courseSettingTypeEnum);

    /**
     * 查询课程权重设置状态
     *
     * @param obeCourseOutlineId 课程大纲id
     * @return 课程权重设置状态
     */
    CourseWeightSettingStatusResult findCourseWeightSettingStatus(Long obeCourseOutlineId);
}